import 'dotenv/config';
import { Injectable, Logger, Module, OnModuleDestroy, OnModuleInit } from '@nestjs/common';
import { AppController } from './app.controller';
import { AppService } from './app.service';
import { GraphQLModule } from '@nestjs/graphql';
import { ApolloDriver, ApolloDriverConfig } from '@nestjs/apollo';
import { join } from 'path';
import { MemberResolver } from './services/reputation/components/member/member.resolver';
import { PostResolver } from './services/reputation/components/post/post.resolver';
import { CitationsResolver } from './services/reputation/components/citation/citation.resolver';

import { LedgerConnector} from './util/ledger-connector';
import { DatabaseConnector } from './util/database-connector';
import { MemberRepository } from './services/reputation/components/member/member.repository';

const dbConnector = new DatabaseConnector(new Logger('DatabaseConnector'));
const ledgerConnector = LedgerConnector.getInstance(new Logger('LedgerConnector'));

@Injectable()
export class StartupService implements OnModuleInit {
  async onModuleInit() {

    const testSuite = process.env.TEST_SUITE;
    if (testSuite?.toLowerCase().includes("integration")) return;

    const dbOptions = {
      host: process.env.DB_HOST,
      port: process.env.DB_PORT,
      user: process.env.DB_USER,
      pass: process.env.DB_PASS
    }

    const ledgerOptions = {
      host: process.env.LEDGER_HOST,
      user: process.env.LEDGER_USER,
      pass: process.env.LEDGER_PASS
    }

    if (!dbOptions.host) throw new Error('Missing environment variable, DB_HOST');
    if (!dbOptions.port) throw new Error('Missing environment variable, DB_PORT');
    if (!dbOptions.user) throw new Error('Missing environment variable, DB_USER');
    if (!dbOptions.pass) throw new Error('Missing environment variable, DB_PASSWORD');

    await dbConnector.connect(dbOptions, [
      MemberRepository
    ]);

    await ledgerConnector.connect(ledgerOptions);

  }
}

@Injectable()
export class ShutdownService implements OnModuleDestroy {
  async onModuleDestroy() {
    await dbConnector.disconnect();
    await ledgerConnector.disconnect();
  }
}

@Module({
  imports: [
    GraphQLModule.forRoot<ApolloDriverConfig>({
      driver: ApolloDriver,
      typePaths: ['./**/*.graphql'],
      definitions: {
        path: join(process.cwd(), 'src/graphql.ts')
      },
    }),
  ],
  controllers: [AppController],
  providers: [
    AppService,
    StartupService,
    ShutdownService,
    MemberResolver,
    PostResolver,
    CitationsResolver
  ],
})

export class AppModule {}
